#include <QtCore>
#include <QtNetwork>
#include <readFile.h>
#include <bundleManager.h>
#include <DTNInterface.h>
#include <tcpClient.h>

#define DTNTIMER 500


#define INIT 0
#define DTNLIST 1
#define IDLE 2
#define PARSE 3

DTNInterface::DTNInterface(BundleManager* bundleMng, NodeManager* nodeMng)
{
	ReadFile conf;
	hostPort=conf.getDTNHostPort();
	hostAddress=conf.getDTNHostName();
	ourName=conf.getNodeName();
	ourId=conf.getNodeId();
	bundleManager = bundleMng;
	nodeManager = nodeMng;
	//Create and connect Hello Timer
	timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(getBundles()));
    timer->start(DTNTIMER);
	client = new TcpClient(this);
    sender = new TcpClient(this);
	connect(client, SIGNAL(newAnswer(QByteArray)), this, SLOT(getResponse(QByteArray)));
	client->addHost(hostAddress.toString(),hostPort);
	sender->addHost(hostAddress.toString(),hostPort);
	dtnState=INIT;
	bundlesToParse=0;
	parsedList.clear();
	bundleList.clear();
	ackOption=conf.getUseACKS();

}

void DTNInterface::receiveBundle(Bundle bundle)
{
	QString command;
	if(bundle.data.size()>0)
	{
		command.append("bundle inject ");
		command.append(bundle.sourceString);
		command.append(" ");
		command.append(bundle.destinationString);
		command.append(" \"");
		for (int m = 0; m < bundle.data.size(); ++m)
		{
			QString ch;
			ch.append("\\x");
			ch.append(QString("%1").arg((unsigned char)bundle.data.at(m),0,16)  );
			command.append(ch);
		}	
		command.append("\"");
		sender->appendCommand(0,command);
		QFile file("inject.txt");
		if(file.open(QIODevice::WriteOnly))
		{
		    file.write(command.toAscii());
		    file.close();
		
		}
		
		emit sendLog("Adding bundle to DTN...");
	}
	else
		emit sendLog("Zero bundle not sent to DTN...");

}


void DTNInterface::getBundles()
{
	if(dtnState==IDLE)
	{
		dtnState=INIT;
		client->appendCommand(0,(QString)"");
	}
}


void DTNInterface::getResponse(QByteArray response)
{
	QString log;
	switch(dtnState)
	{
		case INIT:log.append("INIT:");break;
		case DTNLIST:log.append("DTNLIST:");break;
		case IDLE:log.append("IDLE:");break;
		case PARSE:log.append("PARSE:");break;
	}
	int size=response.size();
	if(size<500)
		log.append(response);
	else
		log.append("Long response...");
	emit sendLog(log);
	QString temp;
	int pos;
	int pos2;
	int inTheList=-1;
	int flag;
	

 	switch(dtnState)
	{
		case INIT:	bundleIndex=0;
					bundleList.clear();
					client->appendCommand(0,(QString)"bundle list");
					dtnState=DTNLIST;
			break;
		case DTNLIST:	//we get the nuber of bundles
						pos = response.indexOf("(",0);
						pos2 = response.indexOf(")",0);
						temp=response.mid(pos+1,pos2-pos-1);
						bundlesToParse=temp.toInt(0,10);
						response.remove(0,pos2+6);
						emit sendLog("NR BUNDLES:");
	   				    emit sendLog(QString("%1").arg(bundlesToParse));
						while(bundlesToParse>0)
						{
							Bundle tempBundle;
							//If acking is enabled
							tempBundle.setContent(bundleManager->getSeq(),ourId,0,"",0);
							pos2 = response.indexOf(":",0);
							temp=response.mid(0,pos2);
							//Parse bundle ID
							tempBundle.options=temp.toInt(0,10);
							response.remove(0,pos2+2);
							//Parse source name
							pos2= response.indexOf("->",0);
							temp=response.mid(0,pos2-1);
							tempBundle.sourceString=temp.toAscii();
							response.remove(0,pos2+3);
							//Parse destination name
							pos2= response.indexOf("length",0);
							temp=response.mid(0,pos2-1);
							tempBundle.destinationString=temp.toAscii();
							//Parse data length
							response.remove(0,pos2+7);
							pos2= response.indexOf("\r",0);
							temp=response.mid(0,pos2);
							tempBundle.dataLength=temp.toInt(0,10);
							//Removing last part
							pos2= response.indexOf("\r",0);
							response.remove(0,pos2+3);
							bundleList.append(tempBundle);
							bundlesToParse--;
						}
						//Checking for unused bundles in parsed list
						for (int m = 0; m < parsedList.size(); ++m)
						{
							inTheList=-1;
							for (int n = 0; n < bundleList.size(); ++n)
							{
								if(parsedList.at(m)==bundleList.at(n).options)
									inTheList=m;
							}
							if(inTheList==-1)
							{
								parsedList.removeAt(m);
								m=0;
							}
						}
						//Check if it is some of the bundles are already on the parsed list
						for (int m = 0; m < parsedList.size(); ++m)
						{
							inTheList=-1;
							for (int n = 0; n < bundleList.size(); ++n)
							{
								if(parsedList.at(m)==bundleList.at(n).options)
									inTheList=n;
							}
							if(inTheList!=-1)
								bundleList.removeAt(inTheList);
						}
						if(bundleList.size()>0)
						{
							QString command;
							command.append("bundle dump ");
							command.append(QString("%1").arg(bundleList.at(bundleIndex).options));
							client->appendCommand(0,command);
							dtnState=PARSE;
							
						}
						else
						{
							dtnState=IDLE;
						}
			break;
		case PARSE:	client->appendCommand(0,(QString)"bundle list");
						//First we extract the bundle payload size
						temp=response.mid(0,20);
						pos=temp.indexOf("Size:",0);			
						int payloadSize;					
						flag=0;
						pos2=temp.indexOf("\n",0);
						temp=temp.mid(pos+5,pos2-(pos+6));
						payloadSize=temp.toInt(0,10);
						if(payloadSize>0)
						{
							
							QByteArray tempData;
							QString hexValue;
							char decValue;
							response.remove(0,pos2+1);
							pos=0;
							while(pos<(2*payloadSize))
							{
								//Decode hex value
								hexValue=response.mid(pos,2);
								decValue=hexValue.toShort(0,16);
								tempData.append(decValue);
								//Move to next value
								pos=pos+2;

							}							
							
								
							Bundle tempBundle=bundleList.at(bundleIndex);
							tempBundle.data=tempData;

						//Removing from the list
							parsedList.append(tempBundle.options);
			
							
							
							
							
							//Checking if this is not our bundle
							QString temp;
							QString destination;
							temp=tempBundle.destinationString;
							pos=temp.indexOf("://",0);
							temp.remove(0,pos+3);
							pos=temp.indexOf("/",0);
							destination=temp.mid(0,pos);
							if(destination!=ourName)
							{
								emit sendLog("Bundle parsed...\r\n");
								tempBundle.options=0;
								//If acking is enabled
								int option=0;
								if(ackOption==1)
								{
									//We set the ACKOption bit and set the ACK bit 
									option=option|0x01;
								}
								tempBundle.options=option;
								if(tempBundle.data.size()>0)
									emit sendBundle(tempBundle);
								bundleManager->updateBundlesDestinations(nodeManager);
							}
							else
							{
								emit sendLog("Bundle parsed but it is for us...\r\n");
							}
						
						}
						bundleIndex++;
						if(bundleIndex<bundleList.size())
						{
							QString command;
							//We need to use bundle dump_tcl command here as well
							command.append("bundle dump_tcl ");
							command.append(QString("%1").arg(bundleList.at(bundleIndex).options));
							client->appendCommand(0,command);
							dtnState=PARSE;
						}
						else
						{
							dtnState=IDLE;
						}
			break;
		case IDLE:		dtnState=IDLE;
			break;
	}

}
