#ifndef BUNDLELISTFILEIO_H
#define BUNDLELISTFILEIO_H
#include <QtCore>

//! Handles the file IO for the QMultiHash that keeps track of which nodes have received certain bundles.
/*! The QMultiHash should have integer bundle ids as keys and a list of 
 * integer node ids as values.
 */
class BundleListFileIO : public QObject
{
	Q_OBJECT

	public:
		BundleListFileIO(QObject *parent = 0);
		BundleListFileIO(const BundleListFileIO& other);
		BundleListFileIO& operator=(const BundleListFileIO& other);;

		//!Creates a new BundleListFileIO object.
		/*!
		 * @param hash the QMultiHash used for storing bundle->node mappings.
		 * @param fileName the name of the file used for storing the hash.
		 */
		BundleListFileIO(QHash<int,int> *hash, QDir storageDir, QString fileName);

		//! Writes the specified item to the file.
		/*! If the key already exists in the file, the old one is overwritten.
		 * @param key the key od what should be written to file.
		 * @return 0 if success, -1 otherwise
		 */
		int writeItem(int key);

		//! Reads all keys and values from file and inserts them into the hash.
		/*! If keys read from file already exist in the hash the values from 
		 * file are appended to the values in the hash.
		 * @return 0 if successful reading file, -1 if file IO error, -2 if parse error.
		 */
		int readAll();

		//! Writes all items in the hash to file
		/*!
		 * @return 0 if all items were written successfully, -1 otherwise. 
		 * Even if -1 is returned, as many items as possible are written to 
		 * file.
		 */
private slots:
		int writeAll();

	private:
		QHash<int,int> *hash;
		QFile* file;
		QTimer* timer;
		int fileOption;
};
#endif
