#include <QtGui>
#include <infoWidget.h>
#include <readFile.h>


#define AVRTIME 1000
#define MAXRATE 100
#define YELLOWONTIME 1000
#define REDONTIME 4000

//Connections states for GUI
#define STATE_SYN 1
#define STATE_SYNACK 2
#define STATE_ACK 3
#define STATE_DICT 4
#define STATE_RIB 5
#define STATE_BOFF 6
#define STATE_BREQ 7
#define STATE_BUNDLE 8
#define STATE_ERROR 9


InfoWidget::InfoWidget(NodeManager* nodeMng,QWidget *parent)
        : QWidget(parent)
{
#ifdef PDAGUI
//	setFixedSize(230,300);
#else
	setFixedSize(990, 660);
#endif //PDAGUI
	ReadFile conf;
	//Traffic timer and variables
	trafficRate=0;
	trafficSum=0;
	trafficTimer = new QTimer();
	yellowTimer = new QTimer();
	yellowTimer->setSingleShot(true);
	redTimer = new QTimer();
	redTimer->setSingleShot(true);

	trafficTimer->start(AVRTIME);
	connect(trafficTimer, SIGNAL(timeout()), this, SLOT(calculateTraffic()));
	connect(yellowTimer, SIGNAL(timeout()), this, SLOT(yellowLedOff()));
	connect(redTimer, SIGNAL(timeout()), this, SLOT(redLedOff()));

	//Load graphic
	redOn = new QPixmap("red_on.png");
	redOff= new QPixmap("red_off.png");
	yellowOn= new QPixmap("yellow_on.png");
	yellowOff= new QPixmap("yellow_off.png");
	greenOn= new QPixmap("green_on.png");
	greenOff= new QPixmap("green_off.png");

#ifndef PDAGUI
	//ConnectionsTable
    QLabel *labelConnections = new QLabel(this);
    labelConnections->setText("Active connections:");
	conTable = new QTableWidget(9, 4, this);
    QStringList horizontalConLabels;
    horizontalConLabels << "ID" << "IP" << "Alive" << "State" ;
	conTable->setHorizontalHeaderLabels(horizontalConLabels); 
#endif //PDAGUI

#ifndef PDAGUI
	//Status group
	QGroupBox *sendBox = new QGroupBox("Status");
#endif //PDAGUI
    //Memory resources
    QLabel *labelMemory = new QLabel(this);
    labelMemory->setText("Memory storage:");
	memoryBar= new QProgressBar;
	memoryBar->setMaximum (conf.getStorageSize());
	memoryBar->setMinimum (0);
	memoryBar->setValue (0);
	memoryBar->setTextVisible(true);
    //File resources
    QLabel *labelFile = new QLabel(this);
    labelFile->setText("File storage:");
	fileBar= new QProgressBar;
	fileBar->setMaximum (conf.getMemoryStorageSize());
	fileBar->setMinimum (0);
	fileBar->setValue (0);
	fileBar->setTextVisible(true);
	//Transfer speed
    QLabel *labelSpeed = new QLabel(this);
    labelSpeed->setText("Transfer speed:");
	speedBar= new QProgressBar;
	speedBar->setMaximum (MAXRATE);
	speedBar->setMinimum (0);
	speedBar->setValue (0);
	speedBar->setTextVisible(true);
	//Red label
	labelRed = new QLabel(this);
    labelRed->setPixmap(*redOff);
	redLine = new QLineEdit(this);
	redLine->setText("No error...");

	//Green label
	labelGreen = new QLabel(this);
    labelGreen->setPixmap(*greenOff);
	greenLine = new QLineEdit(this);
	greenLine->setText("No connection...");

	//Yellow label
	labelYellow = new QLabel(this);
    labelYellow->setPixmap(*yellowOff);
	yellowLine = new QLineEdit(this);
	yellowLine->setText("No transmission...");




	//Final grid layout
    QGridLayout *sendGroup = new QGridLayout;
#ifdef PDAGUI
	sendGroup->setMargin(5);
	sendGroup->setSpacing(3);

	sendGroup->addWidget(labelMemory, 0, 0, 1, 3);
    sendGroup->addWidget(memoryBar, 0, 1, 1, -1);
	sendGroup->addWidget(labelFile, 1, 0, 1, 3);
    sendGroup->addWidget(fileBar, 1, 1, 1, -1);
	sendGroup->addWidget(labelSpeed, 2, 0, 1, 3);
	sendGroup->addWidget(speedBar, 2, 1, 1, -1);
	sendGroup->addWidget(labelRed, 3, 0, 1, 1);	sendGroup->addWidget(redLine, 3, 1, 1, -1);
	sendGroup->addWidget(labelGreen, 4, 0, 1, 1); sendGroup->addWidget(greenLine, 4, 1, 1, -1);
	sendGroup->addWidget(labelYellow, 5, 0, 1, 1); sendGroup->addWidget(yellowLine, 5, 1, 1, -1);
#else
	sendGroup->addWidget(labelMemory, 0, 0);
    sendGroup->addWidget(memoryBar, 1, 0, 1,2);

	sendGroup->addWidget(labelFile, 2, 0);
    sendGroup->addWidget(fileBar, 3, 0, 1,2);
	sendGroup->addWidget(labelSpeed, 4, 0);
	sendGroup->addWidget(speedBar, 5, 0, 1,2);
	sendGroup->addWidget(labelRed, 6, 0);	sendGroup->addWidget(redLine, 6, 1);
	sendGroup->addWidget(labelGreen, 7, 0); sendGroup->addWidget(greenLine, 7, 1 );
	sendGroup->addWidget(labelYellow, 8, 0); sendGroup->addWidget(yellowLine, 8, 1);

#endif //PDAGUI

#ifndef PDAGUI
	sendGroup->addWidget(labelConnections, 9, 0);
	sendGroup->addWidget(conTable, 10, 0,1,2);
#endif //PDAGUI

#ifdef PDAGUI
	//sendGroup->setFixedSize(240, 320);
#else
	sendBox->setFixedSize(450, 635);
	sendBox->setLayout(sendGroup);
#endif //PDAGUI
	///////////////////////////////////////////////////
	QGroupBox *readBox = new QGroupBox("Configuration");

	//Reading configurations
	loadConfiguration();
	//Final grid layout
	QGridLayout *readGroup = new QGridLayout;
	QLabel *labelNODEID = new QLabel("NodeID:");
	readGroup->addWidget(labelNODEID, 0, 0);
	readGroup->addWidget(lineNODEID, 0, 1);
	QLabel *labelNODENAME = new QLabel("Node Name:");
	readGroup->addWidget(labelNODENAME, 1, 0);
	readGroup->addWidget(lineNODENAME, 1, 1);
	QLabel *labelNODEIP = new QLabel("Node IP:");
	readGroup->addWidget(labelNODEIP, 2, 0);
	readGroup->addWidget(lineNODEIP, 2, 1);
	QLabel *labelNODEIP2 = new QLabel("Node IP2:");
	readGroup->addWidget(labelNODEIP2, 3, 0);
	readGroup->addWidget(lineNODEIP2, 3, 1);
	QLabel *labelNODEBROADCAST = new QLabel("Broadcast Address:");
	readGroup->addWidget(labelNODEBROADCAST, 4, 0);
	readGroup->addWidget(lineNODEBROADCAST, 4, 1);
	QLabel *labelBROADCAST = new QLabel("Broadcast Timer:");
	readGroup->addWidget(labelBROADCAST, 5, 0);
	readGroup->addWidget(lineBROADCAST, 5, 1);
	QLabel *labelAGINGTIMER = new QLabel("Aging Timer:");
	readGroup->addWidget(labelAGINGTIMER, 6, 0);
	readGroup->addWidget(lineAGINGTIMER, 6, 1);
	QLabel *labelPENCOUNTER = new QLabel("PEcounter:");
	readGroup->addWidget(labelPENCOUNTER, 7, 0);
	readGroup->addWidget(linePENCOUNTER, 7, 1);
	QLabel *labelBETA = new QLabel("Beta:");
	readGroup->addWidget(labelBETA, 8, 0);
	readGroup->addWidget(lineBETA, 8, 1);
	QLabel *labelGAMMA = new QLabel("Gamma:");
	readGroup->addWidget(labelGAMMA, 9, 0);
	readGroup->addWidget(lineGAMMA, 9, 1);

	QLabel *labelALIVE = new QLabel("Alive Timer:");
	readGroup->addWidget(labelALIVE, 11, 0);
	readGroup->addWidget(lineALIVE, 11, 1);

	QLabel *labelHELLOTIMER = new QLabel("Hello Timer:");
	readGroup->addWidget(labelHELLOTIMER, 12, 0);
	readGroup->addWidget(lineHELLOTIMER, 12, 1);

	QLabel *laberINITIATORTIMER = new QLabel("Initiator Timer:");
//	readGroup->addWidget(laberINITIATORTIMER, 13, 0);
//	readGroup->addWidget(lineINITIATORTIMER, 13, 1);

	QLabel *labelLISTENERTIMER = new QLabel("Listener Timer:");
//	readGroup->addWidget(labelLISTENERTIMER, 14, 0);
//	readGroup->addWidget(lineLISTENERTIMER, 14, 1);

	QLabel *labelDTNHOSTNAME = new QLabel("DTN Host Name:");
	readGroup->addWidget(labelDTNHOSTNAME, 13, 0);
	readGroup->addWidget(lineDTNHOSTNAME, 13, 1);

	QLabel *labelDNTTIMER = new QLabel("DTN Timer:");
	readGroup->addWidget(labelDNTTIMER, 14, 0);
	readGroup->addWidget(lineDNTTIMER, 14, 1);

	QLabel *labelSTORAGESIZE = new QLabel("File Storge size:");
	readGroup->addWidget(labelSTORAGESIZE, 15, 0);
	readGroup->addWidget(lineSTORAGESIZE, 15, 1);

	QLabel *labelMEMORYSIZE = new QLabel("Memory Storge size:");
	readGroup->addWidget(labelMEMORYSIZE, 16, 0);
	readGroup->addWidget(lineMEMORYSIZE, 16, 1);

	QLabel *labelROUTING = new QLabel("Routing Type:");
	readGroup->addWidget(labelROUTING, 17, 0);
	readGroup->addWidget(lineROUTING, 17, 1);

	QLabel *labelQUEUE = new QLabel("Queue Type:");
	readGroup->addWidget(labelQUEUE, 18, 0);
	readGroup->addWidget(lineQUEUE, 18, 1);

	QLabel *labelCONTINIUSUPDATE = new QLabel("Continius update:");
	readGroup->addWidget(labelCONTINIUSUPDATE, 19, 0);
	readGroup->addWidget(lineCONTINIUSUPDATE, 19, 1);

	QLabel *labelSTORAGEPATH = new QLabel("Storage Path:");
	readGroup->addWidget(labelSTORAGEPATH, 20, 0);
	readGroup->addWidget(lineSTORAGEPATH, 20, 1);

	QLabel *labelLOGPATH = new QLabel("Log Path:");
	readGroup->addWidget(labelLOGPATH, 21, 0);
	readGroup->addWidget(lineLOGPATH, 21, 1);

	QLabel *labelMSGPATH = new QLabel("Messages Path:");
	readGroup->addWidget(labelMSGPATH, 22, 0);
	readGroup->addWidget(lineMSGPATH, 22, 1);

	QLabel *labelAGEFILENODES = new QLabel("Age Nodes on Startup:");
	readGroup->addWidget(labelAGEFILENODES, 23, 0);
	readGroup->addWidget(lineAGEFILENODES, 23, 1);

	QLabel *labelUSEFILENODES = new QLabel("Store Nodes To File:");
	readGroup->addWidget(labelUSEFILENODES, 24, 0);
	readGroup->addWidget(lineUSEFILENODES, 24, 1);

	QLabel *labelWRITETOFILETIMER = new QLabel("File Writing Timer:");
	readGroup->addWidget(labelWRITETOFILETIMER, 25, 0);
	readGroup->addWidget(lineWRITETOFILETIMER, 25, 1);

	QLabel *labelUSEFILEBUNDLES = new QLabel("Store Bundels To Files:");
	readGroup->addWidget(labelUSEFILEBUNDLES, 26, 0);
	readGroup->addWidget(lineUSEFILEBUNDLES, 26, 1);


	QLabel *labelLOGGING = new QLabel("Logging:");
	readGroup->addWidget(labelLOGGING, 27, 0);
	readGroup->addWidget(lineLOGGING, 27, 1);

	QLabel *labelUSETTL = new QLabel("Use TTL:");
	readGroup->addWidget(labelUSETTL, 28, 0);
	readGroup->addWidget(lineUSETTL, 28, 1);

	QLabel *labelTTL = new QLabel("TTL:");
	readGroup->addWidget(labelTTL, 29, 0);
	readGroup->addWidget(lineTTL, 29, 1);

	QLabel *labelACKS = new QLabel("Use ACKs:");
	readGroup->addWidget(labelACKS, 30, 0);
	readGroup->addWidget(lineACKS, 30, 1);




 	readBox->setFixedSize(450, 635);
	readBox->setLayout(readGroup);
	Bundle *msg = new Bundle;
	//Define final layout
	QVBoxLayout *layout = new QVBoxLayout;
	layout->setDirection(QBoxLayout::LeftToRight);
#ifdef PDAGUI
    setLayout(sendGroup);
#else
    layout->addWidget(sendBox);
	layout->addWidget(readBox);    
    setLayout(layout);
#endif //PDAGUI
}

void InfoWidget::loadConfiguration()
{
	ReadFile conf;
	lineNODEID = new QLineEdit();
	lineNODEID->setText(QString("%1").arg(conf.getNodeId()));

	lineNODENAME = new QLineEdit();
	lineNODENAME->setText(conf.getNodeName());

	lineNODEIP = new QLineEdit();
	lineNODEIP->setText(conf.getNodeIp().toString());

	lineNODEIP2 = new QLineEdit();
	lineNODEIP2->setText(conf.getNodeIp2().toString());

	lineNODEBROADCAST = new QLineEdit();
	lineNODEBROADCAST->setText(conf.getBroadcast().toString());

	lineBROADCAST = new QLineEdit();
	lineBROADCAST->setText(QString("%1").arg(conf.getBroadcastTimer()));

	lineAGINGTIMER = new QLineEdit();
	lineAGINGTIMER->setText(QString("%1").arg(conf.getAgingTimer()));

	linePENCOUNTER = new QLineEdit();
	linePENCOUNTER->setText(QString("%1").arg(conf.getPEncounter()));

	lineBETA = new QLineEdit();
	lineBETA->setText(QString("%1").arg(conf.getBeta()));

	lineGAMMA = new QLineEdit();
	lineGAMMA->setText(QString("%1").arg(conf.getGamma()));
	
	lineALIVE = new QLineEdit();
	lineALIVE->setText(QString("%1").arg(conf.getAlive()));
	
	lineHELLOTIMER = new QLineEdit();
	lineHELLOTIMER->setText(QString("%1").arg(conf.getHello()));
	
	lineINITIATORTIMER = new QLineEdit();
	lineINITIATORTIMER->setText(QString("%1").arg(conf.getInitiatorTimer()));
	
	lineLISTENERTIMER = new QLineEdit();
	lineLISTENERTIMER->setText(QString("%1").arg(conf.getListenerTimer()));
	
	lineDTNHOSTNAME = new QLineEdit();
	lineDTNHOSTNAME->setText(conf.getDTNHostName().toString());
	
	lineDNTTIMER = new QLineEdit();
	lineDNTTIMER->setText(QString("%1").arg(conf.getDTNTimer()));

	lineSTORAGESIZE = new QLineEdit();
	lineSTORAGESIZE->setText(QString("%1").arg(conf.getStorageSize()));

	lineMEMORYSIZE = new QLineEdit();
	lineMEMORYSIZE->setText(QString("%1").arg(conf.getMemoryStorageSize()));

	lineROUTING = new QLineEdit();
	lineROUTING->setText(QString("%1").arg(conf.getRouting()));

	lineQUEUE = new QLineEdit();
	lineQUEUE->setText(QString("%1").arg(conf.getQueue()));

	lineCONTINIUSUPDATE = new QLineEdit();
	lineCONTINIUSUPDATE->setText(QString("%1").arg(conf.getContiniusUpdate()));

	lineSTORAGEPATH = new QLineEdit();
	lineSTORAGEPATH->setText(QString("%1").arg(conf.getStoragePath()));

	lineLOGPATH = new QLineEdit();
	lineLOGPATH->setText(QString("%1").arg(conf.getLogPath()));

	lineMSGPATH = new QLineEdit();
	lineMSGPATH->setText(QString("%1").arg(conf.getMsgPath()));

	lineAGEFILENODES = new QLineEdit();
	lineAGEFILENODES->setText(QString("%1").arg(conf.getAgeFileNodes()));

	lineUSEFILENODES = new QLineEdit();
	lineUSEFILENODES->setText(QString("%1").arg(conf.getUseFileNodes()));

	lineWRITETOFILETIMER = new QLineEdit();
	lineWRITETOFILETIMER->setText(QString("%1").arg(conf.getWriteToFileTimer()));

	lineUSEFILEBUNDLES = new QLineEdit();
	lineUSEFILEBUNDLES->setText(QString("%1").arg(conf.getUseFileBundles()));

	lineLOGGING = new QLineEdit();
	lineLOGGING->setText(QString("%1").arg(conf.getLogOption()));

	
	lineUSETTL = new QLineEdit();
	lineUSETTL->setText(QString("%1").arg(conf.getUseTTL()));

	lineTTL = new QLineEdit();
	lineTTL->setText(QString("%1").arg(conf.getTTL()));

	lineACKS = new QLineEdit();
	lineACKS->setText(QString("%1").arg(conf.getUseACKS()));

}






void InfoWidget::yellowLedOff()
{
	labelYellow->setPixmap(*yellowOff);
	yellow=false;
	yellowLine->setText("No transmission...");
	yellowTimer->stop();
}

void InfoWidget::redLedOff()
{
	labelRed->setPixmap(*redOff);
	redLine->setText("No error...");
	redTimer->stop();
}



void InfoWidget::getTraffic(int tr)
{
   trafficSum=+tr;
   yellowLine->setText("Transmitting "+QString("%1").arg(tr)+" bytes...");
 
   yellowTimer->start(YELLOWONTIME);//(tr/MAXRATE)*1000);
   labelYellow->setPixmap(*yellowOn);
}

//Calculating the transfer speed in GUI
void InfoWidget::calculateTraffic()
{
	trafficRate=(trafficSum)/(1000.0/AVRTIME);
	trafficSum=0;
	speedBar->setValue (trafficRate);
}


void InfoWidget::getNewConnection(int c)
{
	greenLine->setText("Connection Nr. "+QString("%1").arg(c)+" created...");
	labelGreen->setPixmap(*greenOn);
}


void InfoWidget::getNewConnectionStatus(int c)
{


	switch (c)
	{
	case STATE_SYN:		greenLine->setText("HELLO SYN...");break;
	case STATE_SYNACK:	greenLine->setText("HELLO SYNACK...");break;
	case STATE_ACK:		greenLine->setText("HELLO ACK...");break;
	case STATE_DICT:	greenLine->setText("DICTIONARY...");break;
	case STATE_RIB:		greenLine->setText("RUTING INFORMATION BASE...");break;
	case STATE_BOFF:	greenLine->setText("BUNDLE OFFER...");break;
	case STATE_BREQ:	greenLine->setText("BUNDLE REQUEST...");break;
	case STATE_BUNDLE:	greenLine->setText("BUNDLE DATA...");break;
	case STATE_ERROR:	greenLine->setText("ERROR  STATE...");break;
	default:	greenLine->setText("No connection...");labelGreen->setPixmap(*greenOff);break;
	}
}

void InfoWidget::getRemovedConnection(int c)
{
	redLine->setText("Connection Nr. "+QString("%1").arg(c)+" removed...");
	redTimer->start(REDONTIME);
	labelRed->setPixmap(*redOn);
}

void InfoWidget::getUsedMemoryStorage(int s)
{
	memoryBar->setValue (s);
}

void InfoWidget::getUsedFileStorage(int s)
{
	fileBar->setValue (s);
}


//Receives a list of active connection and add them to the list
void InfoWidget:: getConnectionList(QList <ConnectionInfo> list)
{
#ifdef PDAGUI
	/* send the connection list to the connection tab */
	emit sendConnectionList(list);
#else
	ConnectionInfo tempNode;
	conTable->clear();
        QStringList horizontalConLabels;
	horizontalConLabels << "ID" << "IP" << "Alive" << "State";
	conTable->setHorizontalHeaderLabels(horizontalConLabels);
	for (int i = 0; i < list.size(); ++i)
	{
		tempNode=list.at(i);
		//Adding ID
		QString text = QString("%1").arg((int)tempNode.id);
		QTableWidgetItem *newItem = new QTableWidgetItem(text);
		conTable->setItem(i, 0, newItem);
		//Adding IP
		text = tempNode.ip.toString();
		newItem = new QTableWidgetItem(text);
		conTable->setItem(i, 1, newItem);
		//Adding Alive variable
		text = QString("%1").arg((int)tempNode.alive);
		newItem = new QTableWidgetItem(text);
		conTable->setItem(i, 2, newItem);
		//Adding State of the TCP Connection
		int x = tempNode.tcpSocket->state();
		text = QString("%1").arg((int)x);
		newItem = new QTableWidgetItem(text);
		conTable->setItem(i, 3, newItem);
	}
#endif //PDAGUI
 }
