#ifdef GUI
#include <QtGui>
#include <debugWidget.h>
#endif /*GUI*/
#include <QtCore>
#include <neighbourAwareness.h>
#ifdef DTN_INTERFACE
#include <DTNInterface.h>
#endif /*DTN_INTERFACE*/
#include <messageFile.h>
#include <readFile.h>

using namespace std;


int main(int argc, char *argv[])
{
#ifdef GUI
	QApplication app(argc, argv);
#else
	QCoreApplication app(argc, argv);
#endif /*GUI*/
	ReadFile* conf;
	conf = new ReadFile;
	//Prepare Log File
	if(conf->getLogOption()==1)
	{
		MessageFile *log;
		log = new MessageFile;
		log->set("startup.log", 1000000, 5);
		log->addLog(0,(QString)"Starting Prophet!");
	}

	//Create bundle manager
	BundleManager* mainBundleMananger = new BundleManager(conf->getNodeId());
	//Create node manager
	NodeManager* mainNodeManager = new NodeManager();
	//Neighbor Awarness
	NeighAware* neighbourawareness = new NeighAware(mainBundleMananger,mainNodeManager);
#ifdef DTN_INTERFACE
	//Create DTN interface
	DTNInterface* mainDTNInterface = new DTNInterface(mainBundleMananger,mainNodeManager);
#endif /*DTN_INTERFACE*/
#ifdef GUI
	//Create debug
	DebugWidget* debugWidget = new DebugWidget(mainNodeManager,mainBundleMananger);
	debugWidget->show();
	//Connect Bundle manager, Node manager and Debug widget
	debugWidget->connect(mainBundleMananger,SIGNAL(sendDebugBundleList(QList<Bundle>)),debugWidget,SLOT(getBundleList(QList<Bundle>)));
	debugWidget->connect(debugWidget,SIGNAL(sendAppBundle(Bundle)),mainBundleMananger,SLOT(addBundle(Bundle)));
	debugWidget->connect(mainBundleMananger,SIGNAL(sendFileStorage(int)),debugWidget,SLOT(getUsedFileStorage(int)));
	debugWidget->connect(mainBundleMananger,SIGNAL(sendMemoryStorage(int)),debugWidget,SLOT(getUsedMemoryStorage(int)));
	debugWidget->connect(debugWidget,SIGNAL(removeBundle(int)),mainBundleMananger,SLOT(removeBundle(int)));
	debugWidget->connect(debugWidget,SIGNAL(removeAllBundles()),mainBundleMananger,SLOT(removeAllBundles()));
	//Updating bundle destinations
	debugWidget->connect(mainNodeManager,SIGNAL(updatedNodeList(NodeManager*)),mainBundleMananger,SLOT(updateBundlesDestinations(NodeManager*)));
#ifdef DTN_INTERFACE
	//Connect DTN Inteface and Debug widget
	debugWidget->connect(mainDTNInterface,SIGNAL(sendLog(QString)),debugWidget,SLOT(getLog(QString)));
	mainDTNInterface->connect(mainDTNInterface,SIGNAL(sendBundle(Bundle)),mainBundleMananger,SLOT(addBundle(Bundle)));
	mainDTNInterface->connect(mainBundleMananger,SIGNAL(sendDeliveredBundle(Bundle)),mainDTNInterface,SLOT(receiveBundle(Bundle)));
#endif /*DTN_INTERFACE*/
	//Connect Node manager and Debug widget
	debugWidget->connect(mainNodeManager,SIGNAL(sendDebugNodeList(QList<Node>)),debugWidget,SLOT(getNodeList(QList<Node>)));
	debugWidget->connect(neighbourawareness,SIGNAL(sendDebugConList(QList<ConnectionInfo>)),debugWidget,SLOT(getConnectionList(QList<ConnectionInfo>)));
	debugWidget->connect(neighbourawareness,SIGNAL(traffic(int)),debugWidget,SLOT(getTraffic(int)));
	debugWidget->connect(neighbourawareness, SIGNAL(newConnectionStatus(int)), 	debugWidget, SLOT(getNewConnectionStatus(int)));
	debugWidget->connect(neighbourawareness, SIGNAL(newConnection(int)), 	debugWidget, SLOT(getNewConnection(int)));
	debugWidget->connect(neighbourawareness, SIGNAL(removedConnection(int)), debugWidget, SLOT(getRemovedConnection(int)));
#endif /*GUI*/
	delete conf;
	app.exec();
	return 0;

}
