#include <messageFile.h>
#include <readFile.h>

#ifdef Q_WS_X11
#include <sys/types.h>
#include <sys/stat.h> 
#endif

#include <QTextStream>

void MessageFile::set(QString fileName, int fileSize, int fileNo)
{
	msgFileName = fileName;
	msgFileSize = fileSize;
	msgFileNo = fileNo;
	mutex = new QMutex;
	dt = new QDateTime;
	ReadFile conf;
	storagePath=conf.getLogPath();
	dir.setPath(storagePath);
	//create the directory if it does not exist
	if(!dir.exists())
		dir.mkpath(dir.path());

}


void MessageFile::saveMessage(QString& message)
{
	saveMessage(message, false);
}

void MessageFile::saveMessage(QString& message, bool datetime)
{
	QString dateTime;
	if (datetime == true)
	{
		mutex->lock();
		(*dt) = dt->currentDateTime();
		dateTime = dt->toString("MM/dd/yyyy hh:mm:ss.zzz ");
		mutex->unlock();
	}
	dateTime += message;
	saveMessage(0, dateTime);

}


void MessageFile::saveMessage(int type, QString& message)
{
//	qDebug(message);
	mutex->lock();
	int i = 0;
	bool exists;
	QFile* f;
	f = new QFile(dir.filePath(msgFileName));
	exists = f->exists();
	if (f->open(QIODevice::WriteOnly|QIODevice::Append) == true)
	{
		QTextStream stream(f);
		stream << message;
		stream << "\r\n";
		i = f->size();
//		qDebug(QString("MessageFile size: %1 %2\r\n").arg(i).arg(msgFileSize));
		f->close();
#ifdef Q_WS_X11
		if (exists == false)
			chmod(msgFileName.toAscii(), S_IROTH|S_IWOTH|S_IRGRP|S_IWGRP|S_IRUSR|S_IWUSR);
#endif
	}
	delete f;
	if (i > msgFileSize)
	{
		QDir dir;
		QString fileNameOld;
		QString fileNameNew;
		i = msgFileNo;
		while (i > 1)
		{
			fileNameOld = msgFileName;
			fileNameOld += QString(".%1").arg(i);
			dir.remove(fileNameOld);
			i--;
			fileNameNew = msgFileName;
			fileNameNew += QString(".%1").arg(i);
			dir.rename(fileNameNew, fileNameOld);
		}
		fileNameOld = msgFileName;
		fileNameOld += QString(".%1").arg(i);
		dir.remove(fileNameOld);
		i--;
		fileNameNew = msgFileName;
		dir.rename(fileNameNew, fileNameOld);
	}
	mutex->unlock();
}

void MessageFile::loadList(QStringList& sl, unsigned int maxItemNo)
{
	mutex->lock();
	sl.clear();
	QString line;
	QFile* f;
	f = new QFile(msgFileName);
	if (f->open(QIODevice::ReadOnly) == true)
	{
		QTextStream stream(f);
		while ((!stream.atEnd()) && (maxItemNo > 0))
		{
			sl.prepend(stream.readLine());
			maxItemNo--;
		}
		f->close();
	}
	mutex->unlock();
}

void MessageFile::addLog(int indet,QString message)
{
	QString dateTime;
	QString id;
	id.sprintf(" %03d>> ", indet);
	if (1)
	{
		mutex->lock();
		(*dt) = dt->currentDateTime();
		dateTime = dt->toString("MM/dd/yyyy hh:mm:ss.zzz ");
		mutex->unlock();
	}
	dateTime += id;
	dateTime += message;
	saveMessage(0, dateTime);

}

