#ifndef MESSAGEFILE_H
#define MESSAGEFILE_H

#include <qobject.h>
#include <qfile.h>
#include <qdir.h>
#include <qmutex.h>
#include <qstringlist.h>
#include <qdatetime.h>

//! Handles writing and reading logfiles.
/*!
 * 
 */
class MessageFile : public QObject
{
	Q_OBJECT

public:
	//! Sets the parameters for message file storage.
	/*!
	 * @param fileName the name of the file(s) to store messages in.
	 * @param fileSize when a message file grows to this size, a new file 
	 * is created.
	 * @param fileNo the value the file number counter should be 
	 * initialized to.
	 */
	virtual void set(QString fileName, int fileSize, int fileNo);
	
	//! Reads a list of messages from the message file.
	/*!
	 * @param sl the stringlist where the result is put.
	 * @param maxItemNo the maximum number of items to read from the file.
	 */
	virtual void loadList(QStringList& sl, unsigned int maxItemNo = 0xFFFFFFFF);
	void addLog(int indet,QString message);

public slots:
	//! Saves a message to file. (Internal?) 
	/*!
	 * A new file is created if the old one gets too big.
	 * @param type not used.
	 * @param message the message to be saved to file.
	 */
	virtual void saveMessage(int type, QString& message);
	
	//! Saves a message to file without a timestamp.
	virtual void saveMessage(QString& message);
	
	//! Saves a message to disk with optional timestamp.
	/*!
	 * @param message the message to save to file.
	 * @param datetime if true the current date and time (MM/dd/yyyy 
	 * hh:mm:ss:zzz) will be written along with the message. If false, 
	 * no timestamp is written to file along with the message.
	 */
	virtual void saveMessage(QString& message, bool datetime);

protected:
	QString msgFileName;
	int msgFileSize;
	int msgFileNo;
	QMutex* mutex;
	QDateTime* dt;
	QString storagePath;
	QDir dir;
};

#endif
