#include "tcpClient.h"

TcpClient::TcpClient(QObject* parent) : QTcpSocket(parent)
{

	timer = new QTimer;
	timer->setSingleShot(true);
	connect(timer, SIGNAL(timeout()), this, SLOT(reconnectToHost()));
	connect(this, SIGNAL(stateChanged(QAbstractSocket::SocketState)), this, SLOT(hostStateChanged(QAbstractSocket::SocketState)));
	connect(this, SIGNAL(error(QAbstractSocket::SocketError)), this, SLOT(hostError(QAbstractSocket::SocketError)));
	connect(this, SIGNAL(readyRead()), this, SLOT(hostReadyRead()));
	int i = readBufferSize();
	setReadBufferSize(0);
	currentHostIndex = 0;
}

TcpClient::~TcpClient()
{
}

void TcpClient::addHost(QString hn, int hp)
{
	hostNames << hn;
	hostPorts << hp;
	currentHostIndex = hostNames.count()-1;
	if ((state() == QAbstractSocket::UnconnectedState) && (hostNames.count() == 1))
		connectToHost(hostNames[currentHostIndex], hostPorts[currentHostIndex]);
}

void TcpClient::reconnectToHost()
{
	if (hostNames.count() == 0)
		return;
	currentHostIndex++;
	currentHostIndex = currentHostIndex % hostNames.count();
	connectToHost(hostNames[currentHostIndex], hostPorts[currentHostIndex]);
}

void TcpClient::hostStateChanged(QAbstractSocket::SocketState state)
{
	switch (state)
	{
	case QAbstractSocket::ConnectedState:
		{
			QList<QString>::iterator it;
			it = commands.begin();
			while (it != commands.end())
			{
				write((*it).toAscii());
				it++;
			}
			emit connected();
		}
		break;
	case QAbstractSocket::UnconnectedState:
		timer->start(2000);
		emit disconnected();
		break;
	}
}

void TcpClient::hostError(QAbstractSocket::SocketError error)
{
	abort();
	timer->start(2000);
	emit disconnected();
}

void TcpClient::hostReadyRead()
{
	buffer += readAll();
	int from = 0;
	int idx;
	int size = buffer.size();
	QByteArray end=buffer.mid(size-5);
	idx = end.indexOf("dtn%", from);
	if (idx != -1)
	{
		QByteArray oddan;
		oddan.append(buffer);
		int velikos = oddan.size();
		emit newAnswer(oddan);
		buffer.clear();
	}
}

void TcpClient::appendCommand(int id, QString cmd)
{
	cmd += QString("\r");
	commands.append(cmd);
	if (state() == QAbstractSocket::ConnectedState)
		write(cmd.toAscii());
}

bool TcpClient::isConnected()
{
	if (state() == QAbstractSocket::ConnectedState)
		return true;
	else
		return false;
}
